require 'guitar'

trap('INT') { exit }

commands = %w(
  -help    --help
  -version --version
  console
  practice
  tonic_shape
)

help = <<-HELP
Usage: guitar COMMAND [ARGS]

The available guitar commands are:

    console          Start the console      (short-cut alias: "c")
    practice         Do practice            (short-cut alias: "p")
    tonic_shape      Show tonic shapes      (short-cut alias: "t")

All commands can be run with -h (or --help) for more information.
HELP

if command = commands.detect { |c| c =~ /^#{ARGV.first}/ }
  ARGV.shift

  case command
  when '-help', '--help'
    puts help
  when '-version', '--version'
    puts "guitar #{Guitar::VERSION}"
  when 'console'
    require 'guitar/commands/console'
  when 'practice'
    require 'guitar/commands/practice'
    Guitar::Commands::Practice.new.run
  when 'tonic_shape'
    require 'guitar/commands/tonic_shape'
    Guitar::Commands::TonicShape.new.run
  end
else
  puts "### Error: Command '#{ARGV.first}' can not be recognized ###"
  puts help
end
