require 'optparse'

module Guitar
  module Commands
    class Practice
      def run(args = ARGV)
        options = {}

        parser = OptionParser.new do |opts|
          opts.banner = 'Usage: guitar practice LESSON [OPTIONS]'
          opts.separator ''

          opts.on(
            '-s', '--size=size', Integer,
            'Specifies the guitar fret size.'
          ) { |v| options[:size] = v }

          opts.on(
            '-c', '--color=color', ::String,
            'Specifies the show color.'
          ) { |v| options[:color] = v }

          opts.on(
            '-i', '--interval=interval', Float,
            'Specifies the practice loop interval.'
          ) { |v| options[:interval] = v }

          opts.on(
            '-h', '--help',
            'Show this help message.'
          ) { puts opts; exit }

          opts.separator ''
          opts.separator 'Available LESSONS:'
          opts.separator ''
          opts.separator '     1. Random NOTE(natural).'
          opts.separator '     2. Random NOTE.'
          opts.separator ''
          opts.separator '     3. Random NOTE(natural) in random STRING.'
          opts.separator '     4. Random NOTE in random STRING.'
          opts.separator ''
          opts.separator '     5. Random NOTE(natural) in random FRET(2/5/7/10/12).'
          opts.separator '     6. Random NOTE(natural) in random FRET.'
          opts.separator '     7. Random NOTE in random FRET(2/5/7/10/12).'
          opts.separator '     8. Random NOTE in random FRET.'
          opts.separator ''
          opts.separator '     9. Random NOTE(natural) in FRETBOARD.'
          opts.separator '    10. Random NOTE in FRETBOARD.'
          opts.separator ''
          opts.separator '    11. Mark random NOTE(natural) in FRETBOARD.'
          opts.separator '    12. Mark random NOTE in FRETBOARD.'
        end
        parser.parse!(args)

        case args[0].to_i
        when 1
          options.merge!(natural: true)
          Guitar::Practice.new(options).random_note
        when 2
          Guitar::Practice.new(options).random_note
        when 3
          options.merge!(natural: true)
          Guitar::Practice.new(options).random_note_in_random_string
        when 4
          Guitar::Practice.new(options).random_note_in_random_string
        when 5
          options.merge!(natural: true, fret: [2, 5, 7, 10, 12])
          Guitar::Practice.new(options).random_note_in_random_fret
        when 6
          options.merge!(natural: true)
          Guitar::Practice.new(options).random_note_in_random_fret
        when 7
          options.merge!(fret: [2, 5, 7, 10, 12])
          Guitar::Practice.new(options).random_note_in_random_fret
        when 8
          Guitar::Practice.new(options).random_note_in_random_fret
        when 9
          options.merge!(natural: true)
          Guitar::Practice.new(options).random_note_in_fretboard
        when 10
          Guitar::Practice.new(options).random_note_in_fretboard
        when 11
          options.merge!(natural: true)
          Guitar::Practice.new(options).mark_random_note_in_fretboard
        when 12
          Guitar::Practice.new(options).mark_random_note_in_fretboard
        else
          puts parser
        end
      end
    end
  end
end
