require 'optparse'

module Guitar
  module Commands
    class TonicShape
      def run(args = ARGV)
        size    = 15
        options = {}

        parser = OptionParser.new do |opts|
          opts.banner = 'Usage: guitar tonic_shape TONIC SHAPE [OPTIONS]'
          opts.separator ''
          opts.separator 'TONIC examples:   ["C", "D#", "Ab"]'
          opts.separator 'Available SHAPES: ["<", ">", "/\", "\", "/<"]'
          opts.separator ''

          opts.on(
            '-s', '--size=size', Integer,
            'Specifies the guitar fret size.'
          ) { |v| size = v }

          opts.on(
            '-a', '--as=as', ::String,
            'Specifies the show type [pretty_name, finger, degree].'
          ) { |v| options[:as] = v }

          opts.on(
            '-c', '--color=color', ::String,
            'Specifies the show color.'
          ) { |v| options[:color] = v.split(',') }

          opts.on(
            '-h', '--help',
            'Show this help message.'
          ) { puts opts; exit }
        end
        parser.parse!(args)

        if tonic = args[0]
          f = Guitar::Fretboard.new(size)
          f.desc = "The tonic shape of #{tonic.to_n.name} major".bold
          puts f.show_tonic_shape(tonic, args[1], options).inspect
        else
          puts parser
        end
      end
    end
  end
end
