module Guitar
  class Scale
    attr_reader :tonic, :type, :notes, :steps

    STEPS = {
      :major => [2, 2, 1, 2, 2, 2, 1].freeze,
      :minor => [2, 1, 2, 2, 1, 2, 2].freeze
    }

    STEPS[:maj] = STEPS[:major]
    STEPS[:m]   = STEPS[:minor]

    def initialize(tonic = :C, type = :major)
      type = type.to_sym

      if steps = STEPS[type]
        @type  = type
        @steps = steps
        @tonic = tonic.to_n
      else
        fail "the scale type #{type} is not valid"
      end

      @notes = @steps[0..-2].inject([@tonic]) do |ns, s|
        last = ns.last
        current = last + s
        current.alias! if current.name.to_s[0] == last.name.to_s[0]
        ns << current
      end.freeze
    end

    def degree(note)
      note = note.to_n
      index = @notes.find_index { |n| note === n }
      index && (index + 1)
    end

    def note_names
      @notes.map(&:name)
    end

    def play
      Play.play(*(notes + [tonic + 12]))
    end
  end
end
