require 'socket'
require 'fast_osc'

module Guitar
  class Client
    GUI_ID               = 'GUITAR'
    METHOD_RUN_CODE      = '/run-code'
    METHOD_STOP_ALL_JOBS = '/stop-all-jobs'

    def run_code(code)
      call_server_method(METHOD_RUN_CODE, code)
    end
    alias_method :run, :run_code

    def stop_all_jobs
      call_server_method(METHOD_STOP_ALL_JOBS)
    end
    alias_method :stop, :stop_all_jobs

    def call_server_method(method, *args)
      m = FastOsc.encode_single_message(method, args.unshift(GUI_ID))
      socket.send(m, 0)
    end
    alias_method :call, :call_server_method

    private

    def socket
      @socket ||= UDPSocket.new.tap do |s|
        s.connect(Server::HOST, Server::PORT)
      end
    end
  end
end
