module Guitar
  module Play
    extend self

    def play(*args)
      code = get_code(*args)
      puts code if Guitar.debug?
      client.run_code(code)
    end

    def run(code)
      client.run_code(code)
    end

    def use_bpm(bpm)
      @bpm = bpm
    end

    def use_synth(synth)
      @synth = synth
    end

    def stop
      client.stop_all_jobs
    end

    def client
      @@client ||= Client.new
    end

    private

    def get_code(*args)
      options = ", #{args.pop}" if args.last.is_a?(Hash)

      play_and_sleep = args.map do |x|
        if x.respond_to?(:to_a)
          "play #{x.to_a.map { |y| y.to_n.to_i }}#{options}"
        else
          "play #{x.to_n.to_i}#{options}"
        end
      end.join("\nsleep 1\n")

      uses = ["use_synth :#{@synth || 'pluck'}"]
      uses << "use_bpm #@bpm" if @bpm
      uses = uses.join("\n")

      "#{uses}\n#{play_and_sleep}"
    end
  end
end

[Integer, Symbol, String].each do |klass|
  klass.class_eval do
    def play(options = nil)
      options ? Guitar::Play.play(to_n, options) : Guitar::Play.play(to_n)
    end
  end
end
