require 'socket'

module Guitar
  module Server
    HOST = 'localhost'
    PORT = 4557
    # Sonic Pi uses more than one ports. We nust ensure all the ports are
    # available, or else it can not be started.
    # For example, 4556 (PORT - 1) is scsynth_port,
    # and 4562 (PORT + 5) is osc_midi_in_port.
    PORTS = (PORT - 1)..(PORT + 5)

    SCRIPT_PATHS = [
      '/Applications/Sonic Pi.app/server/bin/sonic-pi-server.rb',
      './app/server/bin/sonic-pi-server.rb',
      '/opt/sonic-pi/app/server/bin/sonic-pi-server.rb',
      '/usr/lib/sonic-pi/server/bin/sonic-pi-server.rb',
    ]

    class << self
      attr_reader :pid

      def start
        if !sonic_pi_installed?
          # TODO Sonic Pi installation instruction
          warn 'Sonic Pi is not installed.'
        elsif running?
          warn 'The Sonic Pi server is running.'
        elsif !ports_available?
          warn 'The Sonic Pi server can not be started, for some ports '\
            'required are in use. You could run Server.stop to close them.'
        else
          @pid = Process.spawn(script, spawn_options)
          print 'Starting sonic-pi server... '
          sleep 1

          success = Process.getpgid(@pid) rescue false
          puts(success ? 'Success' : 'Failure')
        end
      end

      def stop
        PORTS.each do |port|
          `lsof -i :#{port} -t`.strip.split("\n").each do |p|
            Process.kill(:INT, p.to_i)
          end
        end
        @pid = nil
      end

      def sonic_pi_installed?
        SCRIPT_PATHS.any? { |p| File.file?(p) }
      end

      def running?
        !port_available?(PORT)
      end

      def ports_available?
        PORTS.all? { |p| port_available?(p) }
      end

      def script
        "'#{SCRIPT_PATHS.find { |p| File.file?(p) }}'"
      end

      private

      def port_available?(port)
        socket = UDPSocket.new
        socket.bind(HOST, port)
        true
      rescue Errno::EADDRINUSE
        false
      ensure
        socket.close
      end

      def spawn_options
        if Guitar.debug?
          {}
        else
          {in: File::NULL, out: File::NULL, err: File::NULL}
        end
      end
    end
  end
end
